@@ -1729,14 +1729,14 @@ draw_layout(GMozillaEngine *engine, int id, int left, int top, int width, int he
   g_return_if_fail(engine);
   g_return_if_fail(GTK_IS_MOZ_EMBED(engine->engine));
 
-  if (!engine->shmid)
-    return;
-
   MicrobEalXshmRenderer *mshm = (MicrobEalXshmRenderer*)g_object_get_data(G_OBJECT(engine->global), G_MOZILLA_PAINT_LISTENER);
 
   if (!mshm)
     return;
 
+  if (!mshm->mShmData.id)
+    return;
+
   GdkRectangle r = { left, top, width, height };
   mshm->PushUpdateRequest(engine, r, id, zoom);
 }
@@ -3089,6 +3089,9 @@ g_mozilla_engine_window_set_active(GMozillaEngine *engine, int state)
     target->DispatchEvent(event, &defaultActionEnabled);
   }
 
+  nsCOMPtr<nsIDocShell> docShell(do_GetInterface((nsISupports*)webBrowser));
+  NS_ENSURE_TRUE(docShell, FALSE);
+
   EAL_IF_GFREE_FUNC(engine->suspend_timeouts_timer_id, g_source_remove);
   if (active) {
     engine->is_layout_active = TRUE;
@@ -3099,6 +3102,7 @@ g_mozilla_engine_window_set_active(GMozillaEngine *engine, int state)
       engine->timeouts_suspended = FALSE;
     }
     MicrobEalXshmRenderer::ResumePendingUpdates(engine);
+    docShell->SetIsActive(PR_TRUE);
   }
   else {
     engine->is_layout_active = FALSE;
@@ -3107,6 +3111,7 @@ g_mozilla_engine_window_set_active(GMozillaEngine *engine, int state)
     if (suspend_js && !engine->timeouts_suspended)
       engine->suspend_timeouts_timer_id = g_timeout_add(SUSPEND_JS_TIMEOUT, (GSourceFunc)suspend_timeouts_timer_callback, engine);
     MicrobEalXshmRenderer::SuspendPendingUpdates(engine);
+    docShell->SetIsActive(PR_FALSE);
   }
 
   if (!suspend_native)
